unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, Grids;

type
  TTagFound = procedure (ATag: String) of object;
  TTextFound = procedure (AText: String) of object;

  TADIFParser = class
  private
    FText: PChar;
    FOnTagFound: TTagFound;
    FOnTextFound: TTextFound;
  public
    constructor Create(AText: PChar);
    procedure Execute;
    property OnTagFound: TTagFound read FOnTagFound write FOnTagFound;
    property OnTextFound: TTextFound read FOnTextFound write FOnTextFound;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Panel1: TPanel;
    Splitter1: TSplitter;
    StringGrid1: TStringGrid;
    procedure Button1Click(Sender: TObject);
  private
    FCurrRow: Integer;
    FCurrCol: Integer;
    function FindGridColumn(AName: String): Integer;
    procedure TagFoundHandler(ATag: String);
    procedure TextFoundHandler(AText: String);

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
var
  s: String;
  parser: TADIFParser;
begin
  StringGrid1.Columns.Clear;
  FCurrRow := 1;
  s := Memo1.Lines.Text;
  parser := TADIFParser.Create(PChar(s));
  try
    parser.OnTagFound := @TagFoundHandler;
    parser.OnTextFound := @TextFoundHandler;
    parser.Execute;
    StringGrid1.RowCount := FCurrRow + 1;
  finally
    parser.Free;
  end;
end;

function TForm1.FindGridColumn(AName: String): Integer;
var
  i: Integer;
begin
  for i:=0 to StringGrid1.Columns.Count-1 do
    if SameText(StringGrid1.Columns[i].Title.Caption, AName) then begin
      Result := i;
      exit;
    end;

  with StringGrid1.Columns.Add do begin
    Title.Caption := AName;
    Result := Index;
  end;
end;

procedure TForm1.TagFoundHandler(ATag: String);
begin
  if Lowercase(ATag) = 'eor' then begin
    inc(FCurrRow);
    if FCurrRow >= StringGrid1.RowCount then
      StringGrid1.RowCount := StringGrid1.RowCount + 10;
  end
  else
    FCurrCol := FindGridColumn(ATag);
end;

procedure TForm1.TextFoundHandler(AText: String);
begin
  StringGrid1.Cells[FCurrCol+1, FCurrRow] := AText;
end;


constructor TADIFParser.Create(AText: PChar);
begin
  FText := AText;
end;

procedure TADIFParser.Execute;
var
  p: PChar;
  txt: String;
  txtlen: Integer;
  n: Integer;
begin
  if FOnTagFound = nil then
    exit;
  if FOnTextFound = nil then
    exit;

  p := FText;
  while p^ <> #0 do begin
    if p^ = '<' then begin
      txt := '';
      inc(p);
      while (p^ <> #0) and not (p^ in [':', '>']) do begin
        txt := txt + p^;
        inc(p);
      end;
      if p^ = #0 then
        exit;
      FOnTagFound(txt);
      if p^ = ':' then begin
        inc(p);
        txt := '';
        while (p^ <> #0) and (p^ <> '>') do begin
          txt := txt + p^;
          inc(p);
        end;
        n := StrToInt(txt);
      end else begin
        inc(p);
        n := 0;
      end;
      txt := '';
      inc(p);
      while (p^ <> #0) and (n > 0) do begin
        txt := txt + p^;
        dec(n);
        inc(p);
      end;
      FOnTextFound(txt);
    end;
    inc(p);
  end;
end;




end.

